{ TCounter - version 1.0 : Copyright  September 1997 by AM Software : Alexander Meeder

  This simple graphical component can be used to represent any number of 1 to 20
  digits. You simply set its value and it will represent the number in any of five
  predefined ways or the one you add yourself as a bitmap.

}
unit Counter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TCounterType = (ctSmall, ctTraditional, ctTypographical, ctLed, ctGold, ctUserdefined);
  TNumDigits = 1..20;
  TValStr = string[20];

  TCounter = class(TGraphicControl)
  private
    FAutoSize: boolean;
    FCounterType: TCounterType;
    FCounterBmp: TBitmap;
    FNumDigits: TNumDigits;
    FStretch: boolean;
    FTmpBmp: TBitmap;
    FUserBmp: TBitmap;
    FValue: TValStr;

    DigitDims: TRect;

    procedure SetAutoSize(value: boolean);
    procedure SetCounterType(value: TCounterType);
    procedure SetCounterBmp;
    procedure SetNumDigits(value: TNumDigits);
    procedure SetStretch(value: boolean);
    procedure SetUserBmp(value: TBitmap);
    procedure SetValue(value: TValStr);
    function CountWidth: integer;
  protected
    procedure PaintPrefix;
    procedure PaintSuffix;
    procedure Loaded; override;
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property AutoSize: boolean read FAutoSize write SetAutoSize default true;
    property CounterType: TCounterType read FCounterType write SetCounterType default ctTraditional;
    property NumDigits: TNumDigits read FNumDigits write SetNumDigits default 8;
    property Stretch: boolean read FStretch write SetStretch default false;
    property UserBitmap: TBitmap read FUserBmp write SetUserBmp;
    property Value: TValStr read FValue write SetValue;
    property ShowHint;
  end;

procedure Register;

implementation

{$R cnt.res}

procedure TCounter.PaintPrefix;
var
  NumPrefix: word;
  FX: integer;
  DstRect: TRect;
begin
  FTmpBmp.Width := NumDigits*DigitDims.Right;
  FTmpBmp.Height := DigitDims.Bottom;
  FTmpBmp.Canvas.Brush.Color := Color;
  FTmpBmp.Canvas.FillRect(Rect(0,0,Width,Height));

  NumPrefix := NumDigits - Length(Value);
  for FX := 0 to NumPrefix-1 do
  begin
    DstRect := Rect(FX*DigitDims.Right,0,(FX*(DigitDims.Right))+DigitDims.Right, DigitDims.Bottom);
    FTmpBmp.Canvas.CopyRect(DstRect, FCounterBmp.Canvas, DigitDims);
  end;
end;

procedure TCounter.PaintSuffix;
var
  NumSuffix: word;
  NumPrefix: word;
  FX: integer;
  DstRect, SrcRect: TRect;
begin
  NumSuffix := Length(Value);
  NumPrefix := NumDigits-NumSuffix;
  for FX := 1 to NumSuffix do
  begin
    case StrToInt(Copy(Value,FX,1)) of
      0: SrcRect := Rect(0,0,DigitDims.Right,DigitDims.Bottom);
      1: SrcRect := Rect(1*DigitDims.Right, 0, (1*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      2: SrcRect := Rect(2*DigitDims.Right, 0, (2*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      3: SrcRect := Rect(3*DigitDims.Right, 0, (3*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      4: SrcRect := Rect(4*DigitDims.Right, 0, (4*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      5: SrcRect := Rect(5*DigitDims.Right, 0, (5*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      6: SrcRect := Rect(6*DigitDims.Right, 0, (6*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      7: SrcRect := Rect(7*DigitDims.Right, 0, (7*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      8: SrcRect := Rect(8*DigitDims.Right, 0, (8*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
      9: SrcRect := Rect(9*DigitDims.Right, 0, (9*DigitDims.Right)+DigitDims.Right, DigitDims.Bottom);
    end;

    DstRect := Rect(((NumPrefix+FX)-1)*DigitDims.Right,0,(NumPrefix+FX)*(DigitDims.Right), DigitDims.Bottom);
    FTmpBmp.Canvas.CopyRect(DstRect, FCounterBmp.Canvas, SrcRect);
  end;
end;

procedure TCounter.Paint;
begin
  inherited Paint;
  PaintPrefix;
  PaintSuffix;
  if Stretch then
     Canvas.StretchDraw(ClientRect, FTmpBmp)
  else
     Canvas.Draw(0,0,FTmpBmp);
end;

procedure TCounter.Loaded;
begin
  inherited Loaded;
  SetCounterBmp;
end;

function TCounter.CountWidth: integer;
begin
  Result := (FCounterBmp.Width div 10) * FNumDigits;
end;

procedure TCounter.SetAutoSize(value: boolean);
begin
  if value <> FAutoSize then
  begin
    FAutoSize := value;
    if value and (Length(FValue) > NumDigits) then NumDigits := Length(FValue);
  end;
end;

procedure TCounter.SetCounterType(value: TCounterType);
begin
  if value <> FCounterType then
  begin
    if (value = ctUserDefined) and (FUserBmp.Empty = true) then Exit;
    FCounterType := value;
    SetCounterBmp;
    Refresh;
  end;
end;

procedure TCounter.SetCounterBmp;
var
  bmpRes: string;
begin
  case FCounterType of
    ctSmall         : bmpRes := 'SMALL';
    ctTraditional   : bmpRes := 'TRAD';
    ctTypographical : bmpRes := 'TYPO';
    ctLed           : bmpRes := 'LED';
    ctGold          : bmpRes := 'GOLD';
  end;

  if FCounterType <> ctUserdefined then
     FCounterBmp.Handle := LoadBitmap(hInstance, PChar(bmpRes))
  else
     FCounterBmp.Assign(FUserBmp);

  if not FStretch and (csDesigning in ComponentState) then
  begin
    Width := CountWidth;
    Height := FCounterBmp.Height;
  end;
  DigitDims := Rect(0,0,FCounterBmp.Width div 10, FCounterBmp.Height);
end;

procedure TCounter.SetNumDigits(value: TNumDigits);
begin
  if value <> FNumDigits then
  begin
    if (value < Length(FValue)) and FAutoSize then
      FNumDigits := Length(FValue)
    else
      FNumDigits := value;
    if not Stretch then Width := CountWidth;
  end;
end;

procedure TCounter.SetStretch(value: boolean);
begin
  if value <> FStretch then
  begin
    FStretch := value;
    if not Value then
    begin
      Width := NumDigits*DigitDims.Right;
      Height := DigitDims.Bottom;
    end;
    Repaint;
  end;
end;

procedure TCounter.SetUserBmp(value: TBitmap);
begin
  FUserBmp.Assign(value);
  if Assigned(FUserBmp) then CounterType := ctUserdefined;
end;

procedure TCounter.SetValue(value: TValStr);
begin
  if value <> FValue then
  begin
    FValue := value;
    if AutoSize and (Length(Value) > NumDigits) then NumDigits := Length(Value);
    Refresh;
  end;
end;

constructor TCounter.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csOpaque];
  FCounterBmp := TBitmap.Create;
  CounterType := ctTraditional;
  FNumDigits := 8;
  FTmpBmp := TBitmap.Create;
  FUserBmp := TBitmap.Create;
  FAutoSize := true;
  FStretch := false;
  FValue := '0';
  SetCounterBmp;
end;

destructor TCounter.Destroy;
begin
  FCounterBmp.Free;
  FTmpBmp.Free;
  FUserBmp.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TCounter]);
end;

end.
